@@ -15,6 +15,8 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
   # Recognized options:
   #
   # [+:schema+] path to the schema file, relative to Rails.root
+  # [+:table_definition_class_name+] table definition class
+  # (e.g. ActiveRecord::ConnectionAdapters::PostgreSQL::TableDefinition for Postgres) or nil.
   def initialize(config={})
     @log            = StringIO.new
     @logger         = Logger.new(@log)
@@ -23,8 +25,17 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
     @indexes        = Hash.new { |hash, key| hash[key] = [] }
     @schema_path    = config.fetch(:schema){ "db/schema.rb" }
     @config         = config.merge(:adapter => :nulldb)
-    super(nil, @logger)
-    @visitor = Arel::Visitors::ToSql.new self if defined?(Arel::Visitors::ToSql)
+    super *initialize_args
+    @visitor ||= Arel::Visitors::ToSql.new self if defined?(Arel::Visitors::ToSql)
+
+    if config[:table_definition_class_name]
+      ActiveRecord::ConnectionAdapters::NullDBAdapter.send(:remove_const, 'TableDefinition')
+      ActiveRecord::ConnectionAdapters::NullDBAdapter.const_set('TableDefinition',
+        self.class.const_get(config[:table_definition_class_name]))
+    end
+
+    register_types unless NullDB::LEGACY_ACTIVERECORD || \
+                          ActiveRecord::VERSION::MAJOR < 4
   end
 
   # A log of every statement that has been "executed" by this connection adapter
@@ -112,11 +123,19 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
     # NOOP
   end
 
+  def enable_extension(*)
+    # NOOP
+  end
+
   # Retrieve the table names defined by the schema
   def tables
     @tables.keys.map(&:to_s)
   end
 
+  def views
+    [] # TODO: Implement properly if needed - This is new method in rails
+  end
+
   # Retrieve table columns as defined by the schema
   def columns(table_name, name = nil)
     if @tables.size <= 1
@@ -131,12 +150,8 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
 
     if table = @tables[table_name]
       table.columns.map do |col_def|
-        ActiveRecord::ConnectionAdapters::NullDBAdapter::Column.new(
-          col_def.name.to_s,
-          col_def.default,
-          col_def.type,
-          col_def.null
-        )
+        col_args = new_column_arguments(col_def)
+        ActiveRecord::ConnectionAdapters::NullDBAdapter::Column.new(*col_args)
       end
     else
       []
@@ -153,7 +168,7 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
     NullObject.new
   end
 
-  def exec_query(statement, name = 'SQL', binds = [])
+  def exec_query(statement, name = 'SQL', binds = [], options = {})
     self.execution_log << Statement.new(entry_point, statement)
     EmptyResult.new
   end
@@ -181,11 +196,11 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
 
   def delete(statement, name=nil, binds = [])
     with_entry_point(:delete) do
-      super(statement, name)
+      super(statement, name).size
     end
   end
 
-  def select_all(statement, name=nil, binds = [])
+  def select_all(statement, name=nil, binds = [], options = {})
     with_entry_point(:select_all) do
       super(statement, name)
     end
@@ -217,7 +232,7 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
 
   def select(statement, name = nil, binds = [])
     EmptyResult.new.tap do |r|
-      r.columns = columns_for(name)
+      r.bind_column_meta(columns_for(name))
       self.execution_log << Statement.new(entry_point, statement)
     end
   end
@@ -225,8 +240,9 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
   private
 
   def columns_for(table_name)
-    table_def = @tables[table_name]
-    table_def ? table_def.columns : []
+    table_meta = @tables[table_name]
+    return [] unless table_meta
+    table_meta.columns
   end
 
   def next_unique_id
@@ -257,8 +273,16 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
     end
   end
 
+  def includes_column?
+    false
+  end
+
   def new_table_definition(adapter = nil, table_name = nil, is_temporary = nil, options = {})
     case ::ActiveRecord::VERSION::MAJOR
+    when 6
+      TableDefinition.new(self, table_name, temporary: is_temporary, options: options.except(:id))
+    when 5
+      TableDefinition.new(table_name, is_temporary, options.except(:id), nil)
     when 4
       TableDefinition.new(native_database_types, table_name, is_temporary, options)
     when 2,3
@@ -267,4 +291,68 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
       raise "Unsupported ActiveRecord version #{::ActiveRecord::VERSION::STRING}"
     end
   end
+
+  def new_column_arguments(col_def)
+    args_with_optional_cast_type(col_def)
+  end
+
+  def args_with_optional_cast_type(col_def)
+    default_column_arguments(col_def).tap do |args|
+      if defined?(ActiveRecord::ConnectionAdapters::SqlTypeMetadata)
+        meta = ActiveRecord::ConnectionAdapters::SqlTypeMetadata.new(sql_type: col_def.type)
+        args.insert(2, meta_with_limit!(meta, col_def))
+      elsif initialize_column_with_cast_type?
+        args.insert(2, meta_with_limit!(lookup_cast_type(col_def.type), col_def))
+      else
+        args[2] = args[2].to_s + "(#{col_def.limit})" if col_def.limit
+      end
+    end
+  end
+
+  def meta_with_limit!(meta, col_def)
+    meta.instance_variable_set('@limit', col_def.limit)
+    meta
+  end
+
+  def default_column_arguments(col_def)
+    if ActiveRecord::VERSION::MAJOR >= 5
+      [
+        col_def.name.to_s,
+        col_def.default,
+        col_def.null.nil? || col_def.null # cast  [false, nil, true] => [false, true, true], other adapters default to null=true 
+      ]
+    else
+      [
+        col_def.name.to_s,
+        col_def.default,
+        col_def.type,
+        col_def.null.nil? || col_def.null # cast  [false, nil, true] => [false, true, true], other adapters default to null=true 
+      ]
+    end
+  end
+
+  def initialize_column_with_cast_type?
+    ::ActiveRecord::VERSION::MAJOR == 4 && ::ActiveRecord::VERSION::MINOR >= 2
+  end
+
+  def initialize_args
+    return [nil, @logger, @config] if ActiveRecord::VERSION::MAJOR > 3
+    [nil, @logger]
+  end
+
+  # 4.2 introduced ActiveRecord::Type
+  # https://github.com/rails/rails/tree/4-2-stable/activerecord/lib/active_record
+  def register_types
+    if ActiveRecord::VERSION::MAJOR < 5
+      type_map.register_type(:primary_key, ActiveRecord::Type::Integer.new)
+    else      
+      require 'active_model/type'
+      ActiveRecord::Type.register(
+        :primary_key,
+        ActiveModel::Type::Integer,
+        adapter: adapter_name,
+        override: true
+      )
+    end
+  end
 end
