@@ -71,13 +71,13 @@ describe "NullDB" do
     ActiveRecord::Migration.verbose = false
     ActiveRecord::Schema.define do
       create_table(:employees) do |t|
-        t.string  :name
+        t.string  :name, null: false, limit: 50
         t.date    :hire_date
         t.integer :employee_number
         t.decimal :salary
       end
 
-      create_table(:employees_widgets, :id => false) do |t|
+      create_table(:employees_widgets, :id => false, :force => true) do |t|
         t.integer :employee_id
         t.integer :widget_id
       end
@@ -113,6 +113,18 @@ describe "NullDB" do
     should_have_column(Employee, :salary, :decimal)
   end
 
+  it 'should have limit on name' do
+    expect(Employee.columns_hash['name'].limit).to eq 50
+  end
+
+  it "should return true on nullable field" do
+    expect(Employee.columns_hash['salary'].null).to be true
+  end
+
+  it "should return false on non-nullable field" do
+    expect(Employee.columns_hash['name'].null).to be false
+  end
+
   it "should return the appropriate primary key" do
     expect( ActiveRecord::Base.connection.primary_key('employees') ).to eq 'id'
   end
@@ -249,7 +261,7 @@ describe "NullDB" do
 
   def should_have_column(klass, col_name, col_type)
     col = klass.columns_hash[col_name.to_s]
-    expect(col.type).to eq col_type
+    expect(col.sql_type.to_s.gsub(/\([0-9]+\)/, "").to_sym).to eq col_type
   end
 
 
@@ -275,6 +287,10 @@ describe "NullDB" do
     expect( ActiveRecord::Base ).to receive(:connection_pool).and_raise(ActiveRecord::ConnectionNotEstablished)
     expect { NullDB.nullify }.to_not raise_error
   end
+
+  it 'should handle count queries' do 
+    expect(Employee.count).to eql(0)
+  end
 end
 
 # need a fallback db for contextual nullification
@@ -307,3 +323,34 @@ describe NullDB::RSpec::NullifiedDatabase do
     end
   end
 end
+
+
+describe 'adapter-specific extensions' do
+  before(:all) do
+    ActiveRecord::Base.establish_connection :adapter => :nulldb
+    ActiveRecord::Migration.verbose = false
+  end
+
+  it "supports 'enable_extension' in the schema definition" do
+    expect{
+      ActiveRecord::Schema.define do
+        enable_extension "plpgsql"
+      end
+    }.to_not raise_error
+  end
+
+  if ActiveRecord::VERSION::MAJOR > 4
+    it 'registers a primary_key type' do
+      expect(ActiveRecord::Type.lookup(:primary_key, adapter: 'NullDB'))
+        .to be_a(ActiveModel::Type::Integer)
+    end
+  end
+end
+
+describe ActiveRecord::ConnectionAdapters::NullDBAdapter::EmptyResult do
+  it "should return an empty array from #cast_values" do
+    result = described_class.new
+    expect( result.cast_values ).to be_a Array
+    expect( result.cast_values ).to be_empty
+  end
+end
